@@ -14,7 +14,9 @@
 #			added a treeview widget with code to scan and
 #			display /home/usr/MyDocs/.sounds
 # 0.2	2010-03-22	added code to load/save/display prefs, added timeout
-#			bug fixing			
+#			bug fixing
+# 0.3	2010-03-25	fixed some bugs (timeout never did not work)			
+#			added slider
 #
 ##########################################################################
 
@@ -24,22 +26,33 @@ import pygst
 pygst.require("0.10")
 import gst
 import hildon
+from portrait import FremantleRotation
+
+version = "0.3 - 2010-03-29"
+# Enable debugging
+dbg = 0
 
 class GTK_Main:
   def __init__(self):
     # Some global variables are defined here
-    self.dir_l = []      # list to store the directorys containing music files
-    self.dc = 0          # actual number of directory
-    self.file_l = []     # list to store filenames of music files
+    self.pl_dirs = []  	# list to store the directorys containing music files
+    self.dc = 0        	# actual number of directory
+    self.pl_files = [] 	# list to store filenames of music files
+    self.max_files = 0	# number of entries stored in playlist	
     
-    self.play_name = ""
-    self.act_file_id = 0
-    self.max_files = 0
+    self.play_name = ""		#  Name of actual playing audio file
+    self.act_file_id = 0	# id of above
+    self.act_file_len = 0	# len in nano-seconds of above
 
     self.pause_pos = 0;
     self.pause_flag = 0;
 
-    self.prefsfile = "/home/user/MyDocs/.easyplayer/preferences"
+    # Where to look for audio files / books
+    self.base_media_dir = "/home/user/MyDocs/"
+
+    self.prefs_file = "/home/user/MyDocs/.easyplayer/preferences"
+    self.pl_dirs_file = "/home/user/MyDocs/.easyplayer/playlistdirs"
+    self.pl_files_file = "/home/user/MyDocs/.easyplayer/playlistfiles"
 
     self.prefs = {
       "lastfile":0,		# id of last played file
@@ -52,34 +65,61 @@ class GTK_Main:
     }
     
     # Try to load the preferences
-    if os.path.isfile(self.prefsfile):
+    if os.path.isfile(self.prefs_file):
       try:
-        fd = open(self.prefsfile)
+        fd = open(self.prefs_file)
         self.prefs = pickle.load(fd) # overwrite default values
         fd.close()
       except:
-        print "Can't load prefernces"
+        if (dbg): print "Can't load prefernces"
     self.act_file_id = self.prefs["lastfile"]    
     
     if self.prefs["bigbuttons"] == 1: 
       self.std_size = gtk.HILDON_SIZE_AUTO|gtk.HILDON_SIZE_THUMB_HEIGHT
     else:
       self.std_size = gtk.HILDON_SIZE_AUTO|gtk.HILDON_SIZE_FINGER_HEIGHT
-    
-    self.timeout = 0;
-    self.reset_timer() # Reset TimeOut Timer
-    
+        
     program = hildon.Program.get_instance()
     gtk.set_application_name("EasyPlayer")
-   
-    self.base_media_dir = "/home/user/MyDocs/.sounds/"
-    self.rescan(self.base_media_dir)
-	
+
     self.window = hildon.StackableWindow()
+    self.window.show()
+    self.window.connect("destroy", gtk.main_quit, "WM destroy")
+
+    # Try to load the playlist 
+    try:
+      if (dbg): print "Trying to load dirs..."
+      fd = open(self.pl_dirs_file) 	# Load list containing directories
+      self.pl_dirs = pickle.load(fd) 	# overwrite default values
+      fd.close()
+      if (dbg): print "Trying to load files..."      
+      fd = open(self.pl_files_file) 	# Load list containng filenames
+      self.pl_files = pickle.load(fd) # overwrite default values
+      fd.close()
+      self.max_files = len(self.pl_files)	
+      if (dbg): print "(load) Max Files:",self.max_files        
+    except:
+      if (dbg): print "Can't load playlist. Rescanning..."
+      self.window.set_title("Scanning for music files...")
+      hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
+      #banner = hildon.hildon_banner_show_information(self.window, "",\
+      #         "Can't load playlist. Rescanning for music files...")
+      
+      #win = hildon.StackableWindow()
+      #win.set_title("Scanning for audiofiles")
+      #win.show_all()  
+      #if self.window <> None:
+      #  hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
+      self.pl_dirs = []  	# list to store the directorys containing music files
+      self.dc = 0        	# actual number of directory
+      self.pl_files = [] 	# list to store filenames of music files
+      self.max_files = 0	# number of entries stored in playlist	
+      self.rescan(self.base_media_dir)
+      self.save_playlist()
+      if (dbg): print "(save) Max Files:",self.max_files        
+      hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)        
     program.add_window(self.window)
 
-    self.window.set_title("Easy Player")
-    self.window.connect("destroy", gtk.main_quit, "WM destroy")
 
     # Add a menu
     menu = hildon.AppMenu()
@@ -120,10 +160,10 @@ class GTK_Main:
     
     last_dir = -1
     parent = None   	
-    for i in self.file_l:
+    for i in self.pl_files:
       if i[1] > last_dir: # must we add a new parent dir?
         last_dir = i[1]
-        d = self.dir_l[last_dir]
+        d = self.pl_dirs[last_dir]
         if d == self.base_media_dir: # don't add dir if it' base dir
           parent = None
         else: # add a new parent  
@@ -166,13 +206,18 @@ class GTK_Main:
     #vbox.pack_start(self.entry, False, True)
 
     # Create the ProgressBar
+    self.pbar_eb = gtk.EventBox()
+    self.pbar_eb.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+    self.pbar_eb.connect('button-press-event', self.pbar_eb_press_cb)
     self.pbar = gtk.ProgressBar()
-    vbox.pack_start(self.pbar, False, True)
+    self.pbar.set_size_request(-1,85)
+    self.pbar_eb.add(self.pbar)
+    vbox.pack_start(self.pbar_eb, False, True, 0)
     self.pbar.show()
-    self.pbar.set_text("00:00 / 00:00")
-
+    #self.play_name = self.pl_files[self.act_file_id][0]
+    self.pbar.set_text(self.pl_files[self.act_file_id][0]+" - 00:00 / 00:00")
+    
     buttonbox = gtk.HBox()
-
     prev_btn = self.new_button(gtk.STOCK_MEDIA_PREVIOUS)
     prev_btn.connect("clicked", self.prev_cb)
     buttonbox.add(prev_btn)
@@ -208,6 +253,7 @@ class GTK_Main:
     #buttonbox.add(self.lbl_time)
 		
     vbox.pack_start(buttonbox,False,True)	
+    self.window.set_title("Easy Player")
     self.window.show_all()
 		
     self.player = gst.element_factory_make("playbin2", "player")
@@ -220,6 +266,9 @@ class GTK_Main:
     self.time_format = gst.Format(gst.FORMAT_TIME)
     if self.prefs["resumeplay"] == 1:
       self.play_pause_h()
+    self.timeout = 0;
+    self.reset_timer() # Reset TimeOut Timer
+    self.rot_obj = FremantleRotation("EasyPlayer",self.window,"1.0",FremantleRotation.AUTOMATIC)
 
   # Thread to display time / update process bar while running
   # copied from pygst tutorial
@@ -227,12 +276,14 @@ class GTK_Main:
     play_thread_id = self.play_thread_id
     oldtime = time.time()
     gtk.gdk.threads_enter()
-    self.reset_bar()
+    if not self.pause_flag:
+      self.reset_bar()
     gtk.gdk.threads_leave()
     while play_thread_id == self.play_thread_id:
       try:
         time.sleep(0.2)
         dur_int = self.player.query_duration(self.time_format, None)[0]
+	self.act_file_len = dur_int
         dur_str = self.convert_ns(dur_int)
         gtk.gdk.threads_enter()
 	self.pbar.set_text(self.play_name + " - 00:00 / " + dur_str)
@@ -245,36 +296,37 @@ class GTK_Main:
     # Main Thread for displaying time infos etc.
     #
     while play_thread_id == self.play_thread_id:
-      try: 
-        pos_int = self.player.query_position(self.time_format, None)[0]
-        pos_str = self.convert_ns(pos_int)
-        # Do we use timeout for suspend?
-        if self.timeout <> -1:
-          if self.timeout > 0: 
-            newtime = time.time()
-            self.timeout = self.timeout - int(newtime-oldtime)
-            oldtime = newtime
-          else: # Timeout is reached, suspend player
-            self.stop_reset() # prepare stopping of threat
+      if not self.pause_flag:
+        try: 
+          pos_int = self.player.query_position(self.time_format, None)[0]
+          pos_str = self.convert_ns(pos_int)
+          # Do we use timeout for suspend?
+          if self.timeout <> -1:
+            if self.timeout > 0: 
+              newtime = time.time()
+              self.timeout = self.timeout - int(newtime-oldtime)
+              oldtime = newtime
+            else: # Timeout is reached, suspend player
+              self.stop_reset() # prepare stopping of threat
+              gtk.gdk.threads_enter()
+              self.reset_bar()
+              gtk.gdk.threads_leave()
+              self.player.set_state(gst.STATE_NULL)       
+              break; 	# leave threat 
+          if play_thread_id == self.play_thread_id:
             gtk.gdk.threads_enter()
-            self.reset_bar()
+            # don't display time to suspend if we don't use timeour ot we disabled it
+            if (self.timeout == -1) or (self.prefs["displaytimeout"] == 0):
+              self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str)
+            else:
+              to_str = self.convert_s(self.timeout)   
+              self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str + " (" + to_str +")")
+            if dur_int <> 0:  
+              self.pbar.set_fraction(float(pos_int) / float(dur_int))
             gtk.gdk.threads_leave()
-            self.player.set_state(gst.STATE_NULL)       
-            break; 	# leave threat 
-          to_str = self.convert_s(self.timeout)   
-        if play_thread_id == self.play_thread_id:
-          gtk.gdk.threads_enter()
-          # don't display time to suspend if we don't use timeour ot we disabled it
-          if (self.timeout == -1) or (self.prefs["displaytimeout"] == 0):
-            self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str)
-          else:
-            self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str + " -- " + to_str)
-          if dur_int <> 0:  
-            self.pbar.set_fraction(float(pos_int) / float(dur_int))
-          gtk.gdk.threads_leave()
-        time.sleep(1)
-      except:
-        time.sleep(0.2)
+          time.sleep(1)
+        except:
+          time.sleep(0.2)
       
   # Extracs the parent directory name of a given path
   #
@@ -284,13 +336,14 @@ class GTK_Main:
       name = os.path.split(rest)[1] # repeat to get the name
     return name        
 
-  # Resan given dir and subdirs for audio files.
-  #
+  # Recursive rescan given dir and subdirs for audio files.
+  # 
   def rescan(self, dir):
+    if (dbg): print "Scanning dir ",dir
     if not dir.endswith(os.sep):
-      dir = dir + os.sep
-    self.dir_l.append(dir)
-    self.dc = self.dc + 1
+      dir += os.sep
+    self.pl_dirs.append(dir)
+    self.dc += 1
     d = os.listdir(dir)
     for f in d:
       if os.path.isdir(dir+f) == 1: # f is a directory
@@ -299,17 +352,21 @@ class GTK_Main:
         if f.endswith(".mp3") == 1 or \
           f.endswith(".wav") == 1 or \
           f.endswith(".aac") == 1 or \
-          f.endswith(".ogg") == 1:
-            self.file_l.append((f,self.dc-1))
-            self.max_files = self.max_files + 1
+          f.endswith(".ogg") == 1 or \
+          f.endswith(".MP3") == 1 or \
+          f.endswith(".WAV") == 1 or \
+          f.endswith(".AAC") == 1 or \
+          f.endswith(".OGG") == 1:
+            self.pl_files.append((f,self.dc-1))
+            self.max_files += 1
 
   # Rescan dir for music/sound files
   #
   def menu_rescan_cb(self, btn, lbl):
     # clear fileslist
-    self.dir_l = []      # list to store the directorys containing music files
+    self.pl_dirs = []      # list to store the directorys containing music files
     self.dc = 0          # actual number of directory
-    self.file_l = []     # list to store filenames of music files
+    self.pl_files = []     # list to store filenames of music files
     self.max_files = 0
     self.rescan(self.base_media_dir)
 
@@ -332,7 +389,7 @@ class GTK_Main:
   #
   def get_id(self, name):
     for i in range(self.max_files):
-      if name == self.file_l[i][0]:
+      if name == self.pl_files[i][0]:
         return i
     return None      
 
@@ -349,8 +406,18 @@ class GTK_Main:
   # Set process bar to 0 and time/text to 00:00
   #
   def reset_bar(self):
-    self.pbar.set_text("00:00 / 00:00")
+    self.pbar.set_text(self.play_name + " - 00:00 / 00:00")
     self.pbar.set_fraction(0.0)
+
+  # Set process bar to 0 and time/text to 00:00
+  #
+  def update_bar(self, pos, len, frac):
+    if (self.timeout == -1) or (self.prefs["displaytimeout"] == 0):
+      self.pbar.set_text(self.playname + " - " + pos + " / " + len)
+    else:
+      to_str = self.convert_s(self.timeout)   
+      self.pbar.set_text(self.playname + " - " + pos + " / " + len + " ("+to_str+")")
+    self.pbar.set_fraction(frac)
 	
   # Menu "About" callback - display About...
   #
@@ -358,7 +425,7 @@ class GTK_Main:
     note = hildon.hildon_note_new_information(self.window, \
            "EasyPlayer was written 2010 for Maemo 5\n" \
            "by Klaus Rotter (klaus at rotters dot de)\n" \
-           "Version 0.2 - March 2010")   
+           "Version " + version)   
     response = gtk.Dialog.run(note)
     #if response == gtk.RESPONSE_DELETE_EVENT:
     #  print "show_information_note: gtk.RESPONSE_DELETE_EVENT"
@@ -377,9 +444,19 @@ class GTK_Main:
       self.timeout = 30*60	# 30 minutes 
     elif to == 4:
       self.timeout = 60*60	# 60 minutes 
-    elif to > 5:	
+    elif to > 4:	
       self.timeout = -1		# Never
 
+  # Progressbar callback, set new
+  #
+  def pbar_eb_press_cb(self, widget, event):
+    if event.type == gtk.gdk.BUTTON_PRESS:
+      frac = event.x/float(widget.get_allocation().width)
+      self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, 
+		int(self.act_file_len*frac))
+    self.pbar.set_fraction(frac)
+    # FIXME - update progressbar now -> put it in an own function	
+
   # Set Image of Button Play to "Play"
   #  
   def set_play_img(self):
@@ -401,19 +478,20 @@ class GTK_Main:
   # Start / Pause Handler
   #
   def play_pause_h(self):
-    if self.running == False:
+    if self.pause_flag == True:
+        self.set_pause_img()
+        self.player.set_state(gst.STATE_PLAYING)
+        self.pause_flag = None
+    elif self.running == False:
       file_id = self.act_file_id; #which file to play?
-      dir_id = self.file_l[file_id][1]
-      self.play_name = self.file_l[file_id][0]
-      filepath = self.dir_l[dir_id]+self.play_name
+      dir_id = self.pl_files[file_id][1]
+      self.play_name = self.pl_files[file_id][0]
+      filepath = self.pl_dirs[dir_id]+self.play_name
       #print filepath
       if os.path.isfile(filepath):
         self.set_pause_img()
         self.player.set_property("uri", "file://" + filepath)
         self.player.set_state(gst.STATE_PLAYING)
-        if self.pause_flag:
-          self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, self.pause_pos)
-          self.pause_flag = 0;            
 	# Start thread
 	self.play_thread_id = thread.start_new_thread(self.play_thread, ())
         self.running = True
@@ -421,11 +499,12 @@ class GTK_Main:
         self.save_prefs()
     else:
       # We are running, go to pause mode
-      self.pause_pos = self.player.query_position(self.time_format, None)[0]
+      #self.pause_pos = self.player.query_position(self.time_format, None)[0]
       self.pause_flag = True          
-      self.player.set_state(gst.STATE_NULL)
+      self.player.set_state(gst.STATE_PAUSED)
+      self.set_play_img()
       # don't self.reset_bar()
-      self.stop_reset()      
+      #self.stop_reset()      
 
   # "Stop" Button callback - Stop Playback
   #
@@ -456,7 +535,8 @@ class GTK_Main:
     if t == gst.MESSAGE_EOS:
       if self.act_file_id < self.max_files: # Is there at least one file more to play
         self.stop_h() 		# Stop current file if one is playing
-        self.act_file_id = self.act_file_id + 1
+        self.act_file_id += 1
+        #self.act_file_id = self.act_file_id + 1
         self.play_pause_h() 	# Start new file  
       else: 			# No more files, just stop
         self.stop_h()
@@ -464,7 +544,7 @@ class GTK_Main:
     elif t == gst.MESSAGE_ERROR:
       self.stop_h()
       err, debug = message.parse_error()
-      print "Error: %s" % err, debug
+      if (dbg): print "Error: %s" % err, debug
     #else:
     #  print "Msg %d" % t 
 
@@ -499,7 +579,8 @@ class GTK_Main:
     self.reset_timer() # Reset TimeOut Timer
     if self.act_file_id < self.max_files: # Is there at least one file more to play
       self.stop_h() # Stop current file if one is playing
-      self.act_file_id = self.act_file_id + 1
+      self.act_file_id += 1
+      #self.act_file_id = self.act_file_id + 1
       self.play_pause_h() # Start new file  
 
   # Convert a time in secondes to min:sec
@@ -609,18 +690,42 @@ class GTK_Main:
   # Save preferences file
   #
   def save_prefs(self):
-    prefdir = os.path.dirname(self.prefsfile)
+    prefdir = os.path.dirname(self.prefs_file)
     if os.path.exists(prefdir) == 0: # pref dir doesn't exists
       try:
         os.mkdir(prefdir)
       except:
-        print "Can't create preferences directory!"
+        if (dbg):print "Can't create preferences directory!"
     try: 
-      fd = open(self.prefsfile,"w")
+      fd = open(self.prefs_file,"w")
       pickle.dump(self.prefs, fd)
       fd.close()
     except:
-      print "Can't save preferences file"
+      if (dbg):print "Can't save preferences file"
+
+  # Save playlist files: dirs and files
+  #
+  def save_playlist(self):
+    if (dbg):print "Save playlist..."
+    pldir = os.path.dirname(self.pl_dirs_file)
+    if os.path.exists(pldir) == 0: # playlist dir doesn't exists
+      try:
+        os.mkdir(pldir)
+      except:
+        if (dbg):print "Can't create playlist directory!"
+    try: 
+      # Save directories
+      if (dbg):print "Save dirs..."
+      fd = open(self.pl_dirs_file,"w")
+      pickle.dump(self.pl_dirs, fd)
+      fd.close()
+      # Save filenames
+      if (dbg):print "Save filenames..."
+      fd = open(self.pl_files_file,"w")
+      pickle.dump(self.pl_files, fd)
+      fd.close()
+    except:
+      if (dbg):print "Can't save playlist file"
   
   # Helperfunction for makeing a menu item
   #       
